package com.phm.netty.server.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.phm.netty.enums.MessageEnum;
import com.phm.netty.domain.Message;
import com.phm.netty.service.IProcessService;
import com.phm.netty.service.impl.ProcessService;
import com.phm.netty.utils.BitUtils;
import com.phm.netty.utils.ByteUtils;
import com.phm.netty.utils.ChannelMap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.nio.ByteBuffer;

/**
 * @Description ByteArrayMessageHandler
 * @Author WGK
 * @Date 2023/9/20 13:40
 */
@ChannelHandler.Sharable
@Slf4j
@Component
public class ByteArrayMessageHandler extends SimpleChannelInboundHandler<Object> {
    @Autowired
    IProcessService processService;

    private static ByteArrayMessageHandler byteArrayMessageHandler;


    @PostConstruct
    public void init() {
        byteArrayMessageHandler = this;
    }
    /**
     * 设备接入连接时处理
     *
     * @param ctx ctx
     */
    @Override
    public void handlerAdded(ChannelHandlerContext ctx) {
        log.info("有新的连接：[{}]", ctx.channel().id().asLongText());
        Message msg = new Message();
        msg.setData(MessageEnum.SUCCESS.getContent()).setType(MessageEnum.SUCCESS.getType());
        msg.setTarget("Client").setSource("Serve");
        log.info("handlerAdded->Message Info : {}", msg);
        ctx.channel().writeAndFlush(Message.msgToBytes(msg));
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, Object object) throws Exception {
        Message errorMsg = new Message();
        errorMsg.setType(MessageEnum.ERROR.getType());
        // byte[] binaryData = (byte[]) object;
        byte[] binaryData = Message.testMsg();
        // 处理接收到的二进制消息
        int len = 0;
        if (binaryData.length >= 4) {
            // 读取的字节长度
            len = BitUtils.toInt(binaryData, 0);
        }
        if (len <= 0) {
            log.error("解析len属性，条件不满足：{}", len);
            errorMsg.setData("len 格式不对，非正整数");
            ctx.writeAndFlush(Message.msgToBytes(errorMsg));
            return;
        }
        // 创建一个新的字节数组来存储截取的部分
        byte[] subArray = new byte[len];
        // 方法将源字节数组的一部分复制到新数组中
        System.arraycopy(binaryData, 4, subArray, 0, len);
        Message msg = new Message();
        Message.bytesToMsg(msg, subArray, len);
        Message lastMsg = byteArrayMessageHandler.processService.orderHandle(msg);
        if (ObjectUtil.isEmpty(lastMsg)) {
            log.info("Message：{}", lastMsg);
            errorMsg.setData("Message数据为空");
            ctx.writeAndFlush(Message.msgToBytes(errorMsg));
            return;
        }
        log.info("输出传输的内容对象：{}", lastMsg);
        // 转byte[] 返回消息
        ctx.writeAndFlush(Message.msgToBytes(lastMsg));
    }

    /**
     * 设备下线处理
     *
     * @param ctx ctx
     */
    @Override
    public void handlerRemoved(ChannelHandlerContext ctx) {
        log.info("设备下线了:{}", ctx.channel().id().asLongText());
        // map中移除channel
        removeId(ctx);
    }

    /**
     * 设备连接异常处理
     *
     * @param ctx   ctx
     * @param cause cause
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        // 打印异常
        log.info("异常：{}", cause.getMessage());
        // map中移除channel
        removeId(ctx);
        // 关闭连接
        ctx.close();
    }

    private void removeId(ChannelHandlerContext ctx) {
        AttributeKey<String> key = AttributeKey.valueOf("id");
        // 获取channel中id
        String id = ctx.channel().attr(key).get();
        // map移除channel
        if (CollectionUtil.isNotEmpty(ChannelMap.getChannelMap())) {
            ChannelMap.getChannelMap().remove(id);
        } else {
            log.info("ChannelMap === 为空");
        }
    }

    public void channelWrite(Channel channel, Message message) throws Exception {
        if (null == channel) {
            throw new RuntimeException("客户端已离线");
        }
        channel.writeAndFlush(message);
    }
}
